//
// Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
// Copyright (C) - 2013 - Scilab Enterprises - Clement David
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
//

#include <stdlib.h>
#include <string.h>  

#include "api_scilab.h"
#include "Scierror.h"
#include "localization.h"

#include "fmu_wrapper.h"

int fmu_link(char *fname, unsigned long fname_len)
{
    SciErr sciErr;

    int *piAddressLib;
    int *piAddressPrefix;
    int *piAddressFmuType;
    char *varLib;
    char *varPrefix;
    char *varFmuType;
    void *instance;
    FmuModelExchange* instanceMe;
    FmuCoSimulation* instanceCs;
    /*
     * Retrieve the common 3 arguments: library name, library prefix name, fmu type
     */
    if (!checkInputArgument(pvApiCtx, 3, 3))
    {
        return -1;
    }
    sciErr = getVarAddressFromPosition(pvApiCtx, 1, &piAddressLib);
    if (sciErr.iErr)
    {
        printError(&sciErr, 0);
        return sciErr.iErr;
    }
    sciErr = getVarAddressFromPosition(pvApiCtx, 2, &piAddressPrefix);
    if (sciErr.iErr)
    {
        printError(&sciErr, 0);
        return sciErr.iErr;
    }
    sciErr = getVarAddressFromPosition(pvApiCtx, 3, &piAddressFmuType);
    if (sciErr.iErr)
    {
        printError(&sciErr, 0);
        return sciErr.iErr;
    }
    if (getAllocatedSingleString(pvApiCtx, piAddressLib, &varLib))
    {
        return -1;
    }
    if (getAllocatedSingleString(pvApiCtx, piAddressPrefix, &varPrefix))
    {
        free(varLib);
        return -1;
    }
    
    if (getAllocatedSingleString(pvApiCtx, piAddressFmuType, &varFmuType))
    {
        free(varLib);
        return -1;
    }
    /*
     * Retrieve fmu type "me" or "cs" and allocate
     */
    if (!strcmp(varFmuType, "me")) 
    {
        instanceMe = fmu_wrap_model_exchange_new(varLib, varPrefix);
        if (instanceMe == NULL)
        {
            free(varLib);
            free(varPrefix);
            free(varFmuType);
            Scierror(999, fmu_wrap_error(), fname);
            return -1;
        }
        instance = instanceMe;
    }
    else if (!strcmp(varFmuType, "cs")) 
    {
        instanceCs = fmu_wrap_co_simulation_new(varLib, varPrefix);
        if (instanceCs == NULL)
        {
            free(varLib);
            free(varPrefix);
            free(varFmuType);
            Scierror(999, fmu_wrap_error(), fname);
            return -1;
        }
        instance = instanceCs;
    }
    else 
    {
        Scierror(999, _("%s: Wrong type for input argument #%d: FMU type expected.\n"), fname, 3);
        return 0;
    }

    free(varLib);
    free(varPrefix);
    free(varFmuType);

    /*
     * Map the instance to a pointer scilab type
     */
    sciErr = createPointer(pvApiCtx, nbInputArgument(pvApiCtx) + 1, instance);
    if (sciErr.iErr)
    {
        printError(&sciErr, 0);
        return sciErr.iErr;
    }
    AssignOutputVariable(pvApiCtx, 1) = nbInputArgument(pvApiCtx) + 1;
    return 0;
}
