//
// Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
// Copyright (C) - 2013 - Scilab Enterprises - Clement David
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
//

#include <stdlib.h>

#include "api_scilab.h"
#include "Scierror.h"
#include "localization.h"
#include "fmu_wrapper.h"

int fmu_ulink(char *fname, unsigned long fname_len)
{
    SciErr sciErr;
    int *piAddressInstance;
    int *piAddressFmuType;
    void* pInstance;
    char *varFmuType;
    FmuModelExchange* instanceMe;
    FmuCoSimulation* instanceCs;
	
    /*
     * Retrieve 2 arguments: pointer to an instance and the type of FMU (cs/me)
     */
    if (!checkInputArgument(pvApiCtx, 2, 2))
    {
        return -1;
    }
    sciErr = getVarAddressFromPosition(pvApiCtx, 1, &piAddressInstance);
    if (sciErr.iErr)
    {
        printError(&sciErr, 0);
        return sciErr.iErr;
    }
	sciErr = getVarAddressFromPosition(pvApiCtx, 2, &piAddressFmuType);
    if (sciErr.iErr)
    {
        printError(&sciErr, 0);
        return sciErr.iErr;
    }
    sciErr =  getPointer(pvApiCtx, piAddressInstance, &pInstance);
    if (sciErr.iErr)
    {
        printError(&sciErr, 0);
        return sciErr.iErr;
    }
    if (getAllocatedSingleString(pvApiCtx, piAddressFmuType, &varFmuType))
    {
        return -1;
    }
	// check the type of fmu, then free an instance
	if (!strcmp(varFmuType, "me")) 
    {
        instanceMe = fmu_wrap_model_exchange(pInstance);
        if (instanceMe == NULL)
        {
            free(varFmuType);
            Scierror(999, _("%s: Wrong type for input argument #%d: FMU instance for ME expected.\n"), fname, 1);
            return 0;
        }
        fmu_wrap_model_exchange_delete(instanceMe);
    }
    else if (!strcmp(varFmuType, "cs")) 
    {
        instanceCs = fmu_wrap_co_simulation(pInstance);
        if (instanceCs == NULL)
        {
            free(varFmuType);
            Scierror(999, _("%s: Wrong type for input argument #%d: FMU instance for CS expected.\n"), fname, 1);
            return 0;
        }
		fmu_wrap_co_simulation_delete(instanceCs);
    }
    else 
    {
        Scierror(999, _("%s: Wrong type for input argument #%d: FMU type expected.\n"), fname, 2);
        return 0;
    }

    return 0;
}
