/*
 * fmiModelFunctionsTypes.h
 *
 *  Created on: 29 mai 2013
 *      Author: davidcl
 */

#ifndef FMIMODELFUNCTIONSTYPES_H_
#define FMIMODELFUNCTIONSTYPES_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "fmiModelFunctions.h"

    typedef const char* fmiGetModelTypesPlatformTYPE();
    typedef const char* fmiGetVersionTYPE();

    typedef fmiComponent fmiInstantiateModelTYPE(fmiString instanceName, fmiString GUID, fmiCallbackFunctions functions, fmiBoolean loggingOn);

    typedef void      fmiFreeModelInstanceTYPE(fmiComponent c);
    typedef fmiStatus fmiSetDebugLoggingTYPE  (fmiComponent c, fmiBoolean loggingOn);


    /* Providing independent variables and re-initialization of caching */
    typedef fmiStatus fmiSetTimeTYPE                (fmiComponent c, fmiReal time);
    typedef fmiStatus fmiSetContinuousStatesTYPE    (fmiComponent c, const fmiReal x[], size_t nx);
    typedef fmiStatus fmiCompletedIntegratorStepTYPE(fmiComponent c, fmiBoolean* callEventUpdate);
    typedef fmiStatus fmiSetRealTYPE                (fmiComponent c, const fmiValueReference vr[], size_t nvr, const fmiReal    value[]);
    typedef fmiStatus fmiSetIntegerTYPE             (fmiComponent c, const fmiValueReference vr[], size_t nvr, const fmiInteger value[]);
    typedef fmiStatus fmiSetBooleanTYPE             (fmiComponent c, const fmiValueReference vr[], size_t nvr, const fmiBoolean value[]);
    typedef fmiStatus fmiSetStringTYPE              (fmiComponent c, const fmiValueReference vr[], size_t nvr, const fmiString  value[]);


    /* Evaluation of the model equations */
    typedef fmiStatus fmiInitializeTYPE(fmiComponent c, fmiBoolean toleranceControlled,
                                        fmiReal relativeTolerance, fmiEventInfo* eventInfo);

    typedef fmiStatus fmiGetDerivativesTYPE    (fmiComponent c, fmiReal derivatives[]    , size_t nx);
    typedef fmiStatus fmiGetEventIndicatorsTYPE(fmiComponent c, fmiReal eventIndicators[], size_t ni);

    typedef fmiStatus fmiGetRealTYPE   (fmiComponent c, const fmiValueReference vr[], size_t nvr, fmiReal    value[]);
    typedef fmiStatus fmiGetIntegerTYPE(fmiComponent c, const fmiValueReference vr[], size_t nvr, fmiInteger value[]);
    typedef fmiStatus fmiGetBooleanTYPE(fmiComponent c, const fmiValueReference vr[], size_t nvr, fmiBoolean value[]);
    typedef fmiStatus fmiGetStringTYPE (fmiComponent c, const fmiValueReference vr[], size_t nvr, fmiString  value[]);

    typedef fmiStatus fmiEventUpdateTYPE               (fmiComponent c, fmiBoolean intermediateResults, fmiEventInfo* eventInfo);
    typedef fmiStatus fmiGetContinuousStatesTYPE       (fmiComponent c, fmiReal states[], size_t nx);
    typedef fmiStatus fmiGetNominalContinuousStatesTYPE(fmiComponent c, fmiReal x_nominal[], size_t nx);
    typedef fmiStatus fmiGetStateValueReferencesTYPE   (fmiComponent c, fmiValueReference vrx[], size_t nx);
    typedef fmiStatus fmiTerminateTYPE                 (fmiComponent c);

    /* Creation and destruction of slave instances and setting debug status */
    typedef const char* fmiGetTypesPlatformTYPE();
    typedef fmiComponent fmiInstantiateSlaveTYPE (fmiString instanceName, fmiString fmuGUID, fmiString fmuLocation, 
                                                  fmiString mimeType, fmiReal timeout, fmiBoolean visible, fmiBoolean interactive,
                                                  fmiCallbackFunctions functions, fmiBoolean loggingOn);
    typedef fmiStatus fmiInitializeSlaveTYPE	 (fmiComponent c, fmiReal tStart, fmiBoolean StopTimeDefined, fmiReal tStop);
    typedef fmiStatus fmiTerminateSlaveTYPE		 (fmiComponent c);
    typedef fmiStatus fmiResetSlaveTYPE			 (fmiComponent c);
    typedef void      fmiFreeSlaveInstanceTYPE	 (fmiComponent c);
    typedef fmiStatus fmiSetRealInputDerivativesTYPE(fmiComponent c, const fmiValueReference vr[], size_t nvr, const fmiInteger order[], const fmiReal value[]);
    typedef fmiStatus fmiGetRealOutputDerivativesTYPE(fmiComponent c, const fmiValueReference vr[], size_t nvr, const fmiInteger order[], fmiReal value[]);
    typedef fmiStatus fmiCancelStepTYPE			 (fmiComponent c);
    typedef fmiStatus fmiDoStepTYPE				 (fmiComponent c, fmiReal currentCommunicationPoint, fmiReal communicationStepSize, fmiBoolean newStep);
    typedef fmiStatus fmiGetStatusTYPE			 (fmiComponent c, const fmiStatusKind s, fmiStatus*  value);
    typedef fmiStatus fmiGetRealStatusTYPE		 (fmiComponent c, const fmiStatusKind s, fmiReal*    value);
    typedef fmiStatus fmiGetIntegerStatusTYPE	 (fmiComponent c, const fmiStatusKind s, fmiInteger* value);
    typedef fmiStatus fmiGetBooleanStatusTYPE	 (fmiComponent c, const fmiStatusKind s, fmiBoolean* value);
    typedef fmiStatus fmiGetStringStatusTYPE	 (fmiComponent c, const fmiStatusKind s, fmiString*  value);
#ifdef __cplusplus
}  /* end of extern "C" { */
#endif

#endif /* FMIMODELFUNCTIONSTYPES_H_ */
