#include "fmu_wrapper.h"
#include "scicos.h"
#include "scicos_block4.h"
#include "Scierror.h"
#include "scicos_free.h"
#include "sciprint.h"

void free_instance(scicos_block *, int, const char *);

// set input Values
void set_values(scicos_block *block, double *sciInputTypes, size_t inpSize, fmiValueReference *sciInputRefs) {
	
	fmiStatus status;
	int i, j, k;
	fmiReal *valueRealSci = NULL, *valueBoolSci = NULL, *valueReal = NULL;
	fmiInteger *valueIntSci = NULL, valueInt = 0;
	fmiBoolean *valueBool = fmiFalse;
	workFMU *structFmu = *(block->work);
    
    for (i=0, k=1, j=0; i<inpSize; i++)
        {
            if (sciInputTypes[i] == 1) 
            {
                valueRealSci = (double *) block->inptr[i];
                if (sciInputRefs[k] == 1) // real
                {
                    *valueRealSci = -*valueRealSci;
                }
                if (structFmu->type == ModelExchange)
                {
                    status = ((FmuModelExchange*)structFmu->fmu)->fmiSetReal((structFmu->instance), &sciInputRefs[j], 1, valueRealSci);
                }
                else
                {
                    status = ((FmuCoSimulation*)structFmu->fmu)->fmiSetReal((structFmu->instance), &sciInputRefs[j], 1, valueRealSci);
                }
                    if (status > fmiDiscard) 
                    {
                        free_instance(block, -3, "fmiSetReal"); // block produces an internal error
                        return;
                    }
            }
            else if (sciInputTypes[i] == 2) // integer 
            {
                valueIntSci = (int *) block->inptr[i];
                if (sciInputRefs[k] == 1)
                {	
                    *valueIntSci = -*valueIntSci;
                }    
                if (structFmu->type == ModelExchange)
                {
                    status = ((FmuModelExchange*)structFmu->fmu)->fmiSetInteger((structFmu->instance), &sciInputRefs[j], 1, valueIntSci);
                }
                else
                {
                    status = ((FmuCoSimulation*)structFmu->fmu)->fmiSetInteger((structFmu->instance), &sciInputRefs[j], 1, valueIntSci);
                }
                    if (status > fmiWarning) 
                    {
                        free_instance(block, -3, "fmiSetInteger");
                        return;
                    }
            }
            else if (sciInputTypes[i] == 3) // boolean
            {
                valueBoolSci = (double *) block->inptr[i];
                if (sciInputRefs[k] == 1)
                {	
                    if (*valueBoolSci) 
                    {
                        valueBool = (fmiBoolean*)fmiFalse;
                    }	
                    else
                    {
                        valueBool = (fmiBoolean*)fmiTrue;
                    }
                }
                else 
                {
                    if (*valueBoolSci) 
                    {
                        valueBool = (fmiBoolean*)fmiTrue;
                    }	
                    else
                    {
                        valueBool = (fmiBoolean*)fmiFalse;
                    }
                }
                if (structFmu->type == ModelExchange)
                {
                    status = ((FmuModelExchange*)structFmu->fmu)->fmiSetBoolean((structFmu->instance), &sciInputRefs[j], 1, &valueBool);
                }
                else
                {
                    status = ((FmuCoSimulation*)structFmu->fmu)->fmiSetBoolean((structFmu->instance), &sciInputRefs[j], 1, &valueBool);
                }
                    if (status > fmiWarning) 
                    {
                        free_instance(block, -3, "fmiSetBoolean");
                        return;
                    }
            }
        j += 2;
        k += 2;
    }	
} 

// get output Values 
void get_values(scicos_block *block, double *sciOutputTypes, size_t outSize, fmiValueReference *sciOutputRefs) {
	
	fmiStatus status;
	int i, j, k;
	fmiReal *valueRealSci = NULL, *valueBoolSci = NULL, valueReal = 0; 
	fmiInteger *valueIntSci = NULL, valueInt = 0;
	fmiBoolean valueBool = fmiFalse;
	workFMU *structFmu = *(block->work);
    
    for (i=0, k=1, j=0; i<outSize; i++)
        {
            if (sciOutputTypes[i] == 1) 
            {
                valueRealSci = (double *) block->outptr[i];
                if (structFmu->type == ModelExchange)
                {
                    status = ((FmuModelExchange*)structFmu->fmu)->fmiGetReal((structFmu->instance), &sciOutputRefs[j], 1, &valueReal);
                }
                else
                {
                    status = ((FmuCoSimulation*)structFmu->fmu)->fmiGetReal((structFmu->instance), &sciOutputRefs[j], 1, &valueReal);
                    
                }
                    if (status > fmiDiscard)
                    {
                        free_instance(block, -3, "fmiGetReal");
                        return;
                    }
                if (sciOutputRefs[k] == 1)
                {	
                    *valueRealSci = -valueReal; 
                }
                else 
                {
                    *valueRealSci = valueReal;				
                }
            }
            else if (sciOutputTypes[i] == 2) // integer
            {
                valueIntSci = (int *) block->outptr[i];
                if (structFmu->type == ModelExchange)
                {
                    status = ((FmuModelExchange*)structFmu->fmu)->fmiGetInteger((structFmu->instance), &sciOutputRefs[j], 1, &valueInt); 
                }
                else
                {
                    status = ((FmuCoSimulation*)structFmu->fmu)->fmiGetInteger((structFmu->instance), &sciOutputRefs[j], 1, &valueInt);
                    
                }
                    if (status > fmiDiscard)
                    {
                        free_instance(block, -3, "fmiGetInteger");
                        return;
                    }
                if (sciOutputRefs[k] == 1)
                {	
                    *valueIntSci = -valueInt;
                }
                else 
                {
                    *valueIntSci = valueInt;
                }
            }
            else if (sciOutputTypes[i] == 3)
            {
                valueBoolSci = (double *) block->outptr[i];	
                if (structFmu->type == ModelExchange)
                {
                    status = ((FmuModelExchange*)structFmu->fmu)->fmiGetBoolean((structFmu->instance), &sciOutputRefs[j], 1, &valueBool);
                }
                else
                {
                    status = ((FmuCoSimulation*)structFmu->fmu)->fmiGetBoolean((structFmu->instance), &sciOutputRefs[j], 1, &valueBool);
                    
                }
                    if (status > fmiWarning) 
                    {
                        free_instance(block, -3, "fmiGetBoolean");
                        return;
                    }
                
                *valueBoolSci = valueBool;
                if (sciOutputRefs[k] == 1)
                {	
                    if (valueBool) 
                    {
                        *valueBoolSci = fmiFalse;
                    }	
                    else
                    {
                        *valueBoolSci = fmiTrue;
                    }	
                }
                else
                {
                    if (valueBool) 
                    {
                        *valueBoolSci = fmiTrue;
                    }	
                    else
                    {
                        *valueBoolSci = fmiFalse;
                    }
                }
            }
        j += 2;
        k += 2;
    }	

}
void free_instance(scicos_block *block, int errorNumber, const char *functionName)
{
    workFMU *structFmu = *(block->work); 
	sciprint("Error in simulation, returned by %s\n", functionName);
    set_block_error(errorNumber);
    if (structFmu->type == ModelExchange) {
        ((FmuModelExchange*)structFmu->fmu)->fmiFreeModelInstance(structFmu->instance);
    }
    else
    {
        ((FmuCoSimulation*)structFmu->fmu)->fmiFreeSlaveInstance(structFmu->instance);
    }
    scicos_free(structFmu);
	*(block->work) = NULL;
}