//
// Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
// Copyright (C) - 2013 - Scilab Enterprises - Vladislav Trubkin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
//

#include "fmu_wrapper.h"
#include "api_scilab.h"
#include "Scierror.h"
#include "sciprint.h"
#include "scicos.h"
#include "scicos_block4.h"
#include "scicos_malloc.h"
#include "scicos_free.h"

void fmu_simulate_me(scicos_block *block, scicos_flag flag) 
{
    int pathSize = 0, identSize = 0, guidSize = 0, i = 0, j = 0; 
    char *pathName = NULL, *identName = NULL, *guid = NULL;
    double *sciPath = NULL, *sciIdent = NULL, *sciGuid = NULL, *sciDebugLogging = NULL; 
    double *sciTolerance = NULL, *sciInputTypes = NULL, *sciOutputTypes = NULL;
    int *sciNumberOfStates = NULL, *sciNumberOfEvents = NULL; 
    fmiStatus status;
    workFMU *structFmu;
    fmiCallbackFunctions functions;
    double time = 0;
    fmiValueReference *sciRealRefs = NULL, *sciIntRefs = NULL, *sciBoolRefs = NULL; 
    fmiValueReference *sciInputRefs = NULL, *sciOutputRefs = NULL; 
    fmiReal *rpar = NULL, *boolValue = NULL;
    double *valueRealSci = NULL, *valueBoolSci = NULL;
    int *valueIntSci = NULL;
    fmiBoolean loggingOn = fmiFalse;
    fmiInteger *ipar = NULL;
    size_t realRefsSize, intRefsSize, boolRefsSize, outRefsSize, inpRefsSize;
    
    // return if an fmu has not been loaded
    if (block->nopar == 0) return;
    
    // real values
    sciRealRefs = Getuint32OparPtrs(block, 6); // real refs
    realRefsSize = GetOparSize(block, 6, 2); // real refs size
    rpar = GetRparPtrs(block); // real values 
    // integer values
    sciIntRefs = Getuint32OparPtrs(block, 7); 
    intRefsSize = GetOparSize(block, 7, 2); 
    ipar = GetIparPtrs(block); 
    // boolean values
    sciBoolRefs = Getuint32OparPtrs(block, 8); 
    boolRefsSize = GetOparSize(block, 8, 2);  
    boolValue = GetRealOparPtrs(block, 9); // boolean values
    // inputs 
    sciInputRefs = Getuint32OparPtrs(block, 2); 
    inpRefsSize = GetOparSize(block, 2, 2);
    sciInputTypes = GetRealOparPtrs(block, 3); 
    // outputs
    sciOutputRefs = Getuint32OparPtrs(block, 4);
    outRefsSize = GetOparSize(block, 4, 2);
    sciOutputTypes = GetRealOparPtrs(block, 5); 
    // states and events
    sciNumberOfStates = Getuint32OparPtrs(block, 14); 
    sciNumberOfEvents = Getuint32OparPtrs(block, 15);

    if (flag == Initialization) 
    {
        // sciprint("\n");
        // sciprint("*****Initialization block [%d] Time [%f] \n",get_block_number(), get_scicos_time());
        pathSize = GetOparSize(block, 10, 1); // model.opar(10); path for library
        sciPath = GetRealOparPtrs(block, 10); // decimal values
        identSize = GetOparSize(block, 11, 1); 
        sciIdent = GetRealOparPtrs(block, 11); // identifier
        guidSize = GetOparSize(block, 12, 1);
        sciGuid = GetRealOparPtrs(block, 12); // guid
        sciDebugLogging = GetRealOparPtrs(block, 13); // logger
        sciTolerance = GetRealOparPtrs(block, 16); // relative tolerance
        loggingOn = (fmiBoolean)sciDebugLogging[0];
        
        // memory allocation for the main structure 
        if ((*(block->work) = (workFMU*)scicos_malloc(sizeof(workFMU))) == NULL)
        {
            set_block_error(-16); // memory allocation error
            *(block->work) = NULL; 
            return;
        }
        structFmu = *(block->work); 
        // path
        if ((pathName = (char*)scicos_malloc(pathSize * sizeof(char))) == NULL)
        {
            set_block_error(-16);
            scicos_free(structFmu);
            *(block->work) = NULL;
            return;
        }
        else 
        {
        memset(pathName, 0x0, pathSize * sizeof(char));
        }
        for (i=0; i<pathSize; i++)
        {
        pathName[i] = (char)sciPath[i];
        }
        // identifier
        if ((identName = (char*)scicos_malloc(identSize * sizeof(char))) == NULL)
        {
            set_block_error(-16);
            scicos_free(structFmu);
            *(block->work) = NULL;
            return;
        }
        else
        {
            memset(identName, 0x0, identSize * sizeof(char));
        }
        for (i=0; i<identSize; i++)
        {
            identName[i] = (char)sciIdent[i];
        } 
        // guid
        if ((guid = (char*)scicos_malloc(guidSize * sizeof(char))) == NULL)
        {
            set_block_error(-16); 
            scicos_free(structFmu);
            *(block->work) = NULL;
            return;
        }
        else
        {
            memset(guid, 0x0, guidSize * sizeof(char));
        }			
        for (i=0; i<guidSize; i++)
        {
            guid[i] = (char)sciGuid[i];
        } 
        // load library and create an instance 
        if ((structFmu->fmu = (void*)fmu_wrap_model_exchange_new(pathName, identName)) == NULL)
        {
            sciprint(fmu_wrap_error(), "fmu_simulate_me");
            free_instance(block, -3, "fmu_wrap_model_exchange_new"); // internal error
            return;
        }
        // set fmiType
        structFmu->type = ModelExchange;
		// calling helper functions
        functions.logger = fmu_callback_logger;
        functions.allocateMemory = calloc;
        functions.freeMemory = free;
        functions.stepFinished = NULL;
        // instantiate model
        structFmu->instance = ((FmuModelExchange*)structFmu->fmu)->fmiInstantiateModel(identName, guid, functions, loggingOn);
        if (structFmu->instance == NULL) 
        {
            free_instance(block, -3, "fmiInstantiateModel");
            return;
        }
        // set debug logging
        if (loggingOn) 
        {   
            status = ((FmuModelExchange*)structFmu->fmu)->fmiSetDebugLogging(structFmu->instance, fmiTrue);
            if (status > 1)
            {
                free_instance(block, -3, "fmiSetDebugLogging");
                return;
            }
        }
        // set start time
        status = ((FmuModelExchange*)structFmu->fmu)->fmiSetTime((structFmu->instance), get_scicos_time());
        if (status > 1) 
        {
            free_instance(block, -3, "fmiSetTime");
            return;
        }
		// set start values
        // real
        if (realRefsSize > 0)
        {
            if (((structFmu->realRefs) = (fmiValueReference*)scicos_malloc(realRefsSize * sizeof(fmiValueReference))) == NULL)
            {
                free_instance(block, -16, "scicos_malloc");
                return;
            }
            // negated alias
            for (i=0, j=1; i<realRefsSize; i++)
            {
                if (sciRealRefs[j] > 0) 
                {
                    rpar[i] = -rpar[i];
                }
                j += 2;	
            }	
            for (j=0, i=0; i<realRefsSize; i++)
            {
                sciRealRefs[i] = sciRealRefs[j];
                structFmu->realRefs[i] = sciRealRefs[i]; 
                j += 2;
            }
            // set real values	
            status = ((FmuModelExchange*)structFmu->fmu)->fmiSetReal((structFmu->instance), structFmu->realRefs, realRefsSize, rpar); 
            if (status > fmiDiscard) 
            {
                free_instance(block, -3, "fmiSetReal");
                return;
            }
        }	
        // integers
        if (intRefsSize > 0)
        {
            if (((structFmu->intRefs) = (fmiValueReference*)scicos_malloc(intRefsSize * sizeof(fmiValueReference))) == NULL)
            {
                free_instance(block, -16, "scicos_malloc");
                return;
            }	
            // negated alias
            for (i=0, j=1; i<intRefsSize; i++)
            {
                if (sciIntRefs[j] > 0) 
                {
                    ipar[i] = -ipar[i];
                }
                j += 2;	
            }
            for (j=0, i=0; i<intRefsSize; i++)
            {
                sciIntRefs[i] = sciIntRefs[j];
                structFmu->intRefs[i] = sciIntRefs[i]; 
                j += 2;
            }
            // set integer values	
            status = ((FmuModelExchange*)structFmu->fmu)->fmiSetInteger((structFmu->instance), structFmu->intRefs, intRefsSize, ipar); 
            if (status > fmiWarning) 
            {
                free_instance(block, -3, "fmiSetInteger");
            }
        }
        // boolean
        if (boolRefsSize > 0)
        {
            if (((structFmu->boolValue) = (fmiBoolean*)scicos_malloc(boolRefsSize * sizeof(fmiBoolean))) == NULL)
            {
                free_instance(block, -16, "scicos_malloc");
                return;
            }	
            for (i=0, j=1; i<boolRefsSize; i++)
            {
                if (sciBoolRefs[j] > 0) 
                {
                    // negated alias
                    if (boolValue[i] == 1) 
                    {
                        structFmu->boolValue[i] = fmiFalse;
                    }
                    else
                    {
                        structFmu->boolValue[i] = fmiTrue;
                    }
                }
                else 
                {
                    if (boolValue[i] == 1) 
                    {
                        structFmu->boolValue[i] = fmiTrue;
                    }
                    else
                    {
                        structFmu->boolValue[i] = fmiFalse;
                    }
                }
                j += 2;	
            }
            if (((structFmu->boolRefs) = (fmiValueReference*)scicos_malloc(boolRefsSize * sizeof(fmiValueReference))) == NULL)
            {
                free_instance(block, -16, "scicos_malloc");
                return;
            }	
            for (j=0, i=0; i<boolRefsSize; i++)
            {
                sciBoolRefs[i] = sciBoolRefs[j];
                structFmu->boolRefs[i] = sciBoolRefs[i]; 
                j += 2;
            }
            // set boolean values	
            status = ((FmuModelExchange*)structFmu->fmu)->fmiSetBoolean((structFmu->instance), structFmu->boolRefs, boolRefsSize, structFmu->boolValue);
            if (status > fmiWarning)
            {
                free_instance(block, -3, "fmiSetBoolean");
                return;
            }
        }
        // initialize model 
        if (sciTolerance[0]) 
        {
            status = ((FmuModelExchange*)structFmu->fmu)->fmiInitialize((structFmu->instance), fmiTrue, 1.0E-06, &structFmu->eventInfo);
            if (status > fmiWarning)
            {
                free_instance(block, -3, "fmiInitialize");
                return;
            }
        }
        else
        {
            status = ((FmuModelExchange*)structFmu->fmu)->fmiInitialize((structFmu->instance), fmiFalse, 0.0, &structFmu->eventInfo);
            if (status > fmiWarning)
            {
                free_instance(block, -3, "fmiInitialize");
                return;
            }
        }
        
        if (sciNumberOfStates[0] > 0) { 
            structFmu->states = (double*)scicos_malloc(sciNumberOfStates[0] * sizeof(double));
            structFmu->nominalStates = (double*)scicos_malloc(sciNumberOfStates[0] * sizeof(double));
            structFmu->derivatives = (double*)scicos_malloc(sciNumberOfStates[0] * sizeof(double));
            structFmu->refsStates = (fmiValueReference*) scicos_malloc(sciNumberOfStates[0] * sizeof(fmiValueReference));
            if (!structFmu->states || !structFmu->nominalStates || !structFmu->derivatives || !structFmu->refsStates)
            {
                free_instance(block, -16, "scicos_malloc");
                return;
            }	
        }
        
        status = ((FmuModelExchange*)structFmu->fmu)->fmiGetContinuousStates((structFmu->instance), structFmu->states, sciNumberOfStates[0]);
        if (status > 1) 
        {
            free_instance(block, -3, "fmiGetContinuousStates");
        }
        for (i=0; i<sciNumberOfStates[0]; i++)
        {
            block->x[i] = structFmu->states[i];
        }
        
        status = ((FmuModelExchange*)structFmu->fmu)->fmiGetStateValueReferences((structFmu->instance), structFmu->refsStates, sciNumberOfStates[0]);
        if (status > 1) 
        {
            free_instance(block, -3, "fmiGetStateValueReferences");
        }
        
        if (sciNumberOfEvents[0] > 0)
        {	
            structFmu->events = (double*)scicos_malloc(sciNumberOfEvents[0] * sizeof(double));
            structFmu->tmpevents = (double*)scicos_malloc(sciNumberOfEvents[0] * sizeof(double));
            if (!structFmu->events || !structFmu->tmpevents)
            {
                free_instance(block, -16, "scicos_malloc");
                return;
            }
            status = ((FmuModelExchange*)structFmu->fmu)->fmiGetEventIndicators((structFmu->instance), structFmu->events, sciNumberOfEvents[0]);
            if (status > fmiDiscard)
            {
                free_instance(block, -3, "fmiGetEventIndicators");
                return;
            }
        }
        // retrieve solution at time = 0.00
        get_values(block, sciOutputTypes, outRefsSize, sciOutputRefs);
    }
	
    else if (flag == DerivativeState)
    {
        // sciprint("\n");
        // sciprint("*****Derivative State block [%d] at time: %f\n",get_block_number(), get_scicos_time());
        if ((structFmu = *(block->work)) == NULL) return;
        
        structFmu->callEventUpdate = fmiFalse;
        
        // set time 
        time = get_scicos_time();
        status = ((FmuModelExchange*)structFmu->fmu)->fmiSetTime((structFmu->instance), time);
        if (status > fmiWarning) 
        {
            free_instance(block, -3, "fmiSetTime");
            return;
            
        }
        
        // set inputs 
        set_values(block, sciInputTypes, inpRefsSize, sciInputRefs);	
        for (i=0; i<sciNumberOfStates[0]; i++)
        {
            structFmu->states[i] = block->x[i];
        }
            status = ((FmuModelExchange*)structFmu->fmu)->fmiSetContinuousStates((structFmu->instance), structFmu->states, sciNumberOfStates[0]);
            if (status > fmiDiscard)
            {
                free_instance(block, -3, "fmiSetContinuousStates");
                return;
            }
            status = ((FmuModelExchange*)structFmu->fmu)->fmiCompletedIntegratorStep((structFmu->instance), &structFmu->callEventUpdate);
            if (status > fmiWarning)
            {
                free_instance(block, -3, "fmiCompletedIntegratorStep");
                return;
            }
            status = ((FmuModelExchange*)structFmu->fmu)->fmiGetDerivatives((structFmu->instance), structFmu->derivatives, sciNumberOfStates[0]);
            if (status > fmiDiscard)
            {
                free_instance(block, -3, "fmiGetDerivatives");
                return;
            }
            for (i=0; i<sciNumberOfStates[0]; i++)
            {
                block->xd[i] = structFmu->derivatives[i];
            }
        get_values(block, sciOutputTypes, outRefsSize, sciOutputRefs);
    }
	
    else if (flag == OutputUpdate)
    {
        // sciprint("\n");
        // sciprint("*****Output update block [%d] at time: %f\n", get_block_number(), get_scicos_time());
        if ((structFmu = *(block->work)) == NULL) return;
        
        if (structFmu->eventInfo.terminateSimulation != fmiTrue) 
        {
            structFmu->timeEvent = fmiFalse;
            structFmu->stateEvent = fmiFalse;
            
            time = get_scicos_time();
            status = ((FmuModelExchange*)structFmu->fmu)->fmiSetTime((structFmu->instance), time);
            if (status > fmiWarning) 
            {
                free_instance(block, -3, "fmiSetTime");
                return;
            }
            set_values(block, sciInputTypes, inpRefsSize, sciInputRefs);
            if (sciNumberOfEvents[0] > 0)
            {
                for (i=0; i<sciNumberOfEvents[0]; i++)
                {
                    structFmu->tmpevents[i] = structFmu->events[i];
                }
                status = ((FmuModelExchange*)structFmu->fmu)->fmiGetEventIndicators((structFmu->instance), structFmu->events, sciNumberOfEvents[0]);
                if (status > fmiDiscard)
                {
                    free_instance(block, -3, "fmiGetEventIndicators");
                    return;
                }
            }
            
            structFmu->timeEvent = (structFmu->eventInfo.upcomingTimeEvent) && (structFmu->eventInfo.nextEventTime <= time);
            
            for (i=0; i<sciNumberOfEvents[0]; i++)
            {			
                if (structFmu->tmpevents[i] * structFmu->events[i] < 0) 
                {
                    structFmu->stateEvent = fmiTrue;
                    break;
                }
            }
            
            if (structFmu->timeEvent || structFmu->stateEvent || structFmu->callEventUpdate)
            {
                structFmu->eventInfo.iterationConverged = fmiFalse;
                status = ((FmuModelExchange*)structFmu->fmu)->fmiEventUpdate((structFmu->instance), fmiFalse, &structFmu->eventInfo);
                if (status > fmiWarning)
                {
                    free_instance(block, -3, "fmiEventUpdate");
                    return;
                }
                    if (structFmu->eventInfo.iterationConverged == fmiTrue) 
                    {
                        get_values(block, sciOutputTypes, outRefsSize, sciOutputRefs);
                    }
                if (structFmu->eventInfo.stateValuesChanged == fmiTrue)
                {
                    status = ((FmuModelExchange*)structFmu->fmu)->fmiGetContinuousStates((structFmu->instance), structFmu->states, sciNumberOfStates[0]);
                    if (status > fmiWarning)
                    {
                        free_instance(block, -3, "fmiGetContinuousStates");
                        return;
                    }
                        for (i=0; i<sciNumberOfStates[0]; i++)
                        {
                            block->x[i] = structFmu->states[i];
                        }
                }
                if (structFmu->eventInfo.stateValueReferencesChanged == fmiTrue)
                {
                    status = ((FmuModelExchange*)structFmu->fmu)->fmiGetNominalContinuousStates((structFmu->instance), structFmu->nominalStates, sciNumberOfStates[0]);
                    if (status > fmiWarning)
                    {
                        free_instance(block, -3, "fmiGetNominalContinuousStates");
                        return;
                    }
                }
            }
            
            // get outputs
            get_values(block, sciOutputTypes, outRefsSize, sciOutputRefs);
        }
    }
    else if (flag == Ending)
    {
        // sciprint("\n");
        // sciprint("*****Ending block [%d] \n", get_block_number());
        if ((structFmu = *(block->work)) == NULL) return;	 
        
        time = get_scicos_time();
        status = ((FmuModelExchange*)structFmu->fmu)->fmiSetTime((structFmu->instance), time);
        if (status > fmiWarning)
        {
            free_instance(block, -3, "fmiSetTime");
            return;
        }
        
        // terminate simulation
        status = ((FmuModelExchange*)structFmu->fmu)->fmiTerminate(structFmu->instance);
        if (status > fmiWarning)
        {
            free_instance(block, -3, "fmiTerminate");
            return;
        }
        
        get_values(block, sciOutputTypes, outRefsSize, sciOutputRefs);
        
        // freeing of instance
        ((FmuModelExchange*)structFmu->fmu)->fmiFreeModelInstance(structFmu->instance);
        // unload library and delete all
        fmu_wrap_model_exchange_delete((FmuModelExchange*)structFmu->fmu);
        scicos_free(structFmu);
    }
}