#ifndef FMU_WRAPPER_H
#define FMU_WRAPPER_H

#include "fmiModelTypes.h"
#include "fmiModelFunctions.h"
#include "fmiModelFunctionsTypes.h"
#include "localization.h"

/**
 * do not use any model identifier
 */
#define MODEL_IDENTIFIER

/**
 * Model Exchange functions
 */
typedef struct
{
    /** constant descriptor used to check for a valid pointer type */
    const char* klass;
    /** name of the FMU */
    char* klassName;
    /** library handle used to map the symbols */
    void* sharedLibraryHandle;

#define FMI_KLASS_METHOD(fun) fun##TYPE* fun
    FMI_KLASS_METHOD(fmiGetModelTypesPlatform);
    FMI_KLASS_METHOD(fmiGetVersion);
    FMI_KLASS_METHOD(fmiInstantiateModel);
    FMI_KLASS_METHOD(fmiFreeModelInstance);
    FMI_KLASS_METHOD(fmiSetDebugLogging);
    FMI_KLASS_METHOD(fmiSetTime);
    FMI_KLASS_METHOD(fmiSetContinuousStates);
    FMI_KLASS_METHOD(fmiCompletedIntegratorStep);
    FMI_KLASS_METHOD(fmiSetReal);
    FMI_KLASS_METHOD(fmiSetInteger);
    FMI_KLASS_METHOD(fmiSetBoolean);
    FMI_KLASS_METHOD(fmiSetString);
    FMI_KLASS_METHOD(fmiInitialize);
    FMI_KLASS_METHOD(fmiGetDerivatives);
    FMI_KLASS_METHOD(fmiGetEventIndicators);
    FMI_KLASS_METHOD(fmiGetReal);
    FMI_KLASS_METHOD(fmiGetInteger);
    FMI_KLASS_METHOD(fmiGetBoolean);
    FMI_KLASS_METHOD(fmiGetString);
    FMI_KLASS_METHOD(fmiEventUpdate);
    FMI_KLASS_METHOD(fmiGetContinuousStates);
    FMI_KLASS_METHOD(fmiGetNominalContinuousStates);
    FMI_KLASS_METHOD(fmiGetStateValueReferences);
    FMI_KLASS_METHOD(fmiTerminate);
#undef FMI_KLASS_METHOD
} FmuModelExchange;

typedef struct
{
    /** constant descriptor used to check for a valid pointer type */
    const char* klass;
    /** name of the FMU */
    char* klassName;
    /** library handle used to map the symbols */
    void* sharedLibraryHandle;
    
#define FMI_KLASS_METHOD(fun) fun##TYPE* fun
    FMI_KLASS_METHOD(fmiGetTypesPlatform);
    FMI_KLASS_METHOD(fmiGetVersion);
    FMI_KLASS_METHOD(fmiSetDebugLogging);
    FMI_KLASS_METHOD(fmiSetReal);
    FMI_KLASS_METHOD(fmiSetInteger);
    FMI_KLASS_METHOD(fmiSetBoolean);
    FMI_KLASS_METHOD(fmiSetString);
    FMI_KLASS_METHOD(fmiGetReal);
    FMI_KLASS_METHOD(fmiGetInteger);
    FMI_KLASS_METHOD(fmiGetBoolean);
    FMI_KLASS_METHOD(fmiGetString);
    FMI_KLASS_METHOD(fmiInstantiateSlave);
    FMI_KLASS_METHOD(fmiInitializeSlave);
    FMI_KLASS_METHOD(fmiTerminateSlave);
    FMI_KLASS_METHOD(fmiResetSlave);
    FMI_KLASS_METHOD(fmiFreeSlaveInstance);
    FMI_KLASS_METHOD(fmiSetRealInputDerivatives);
    FMI_KLASS_METHOD(fmiGetRealOutputDerivatives);
    FMI_KLASS_METHOD(fmiDoStep);
    FMI_KLASS_METHOD(fmiCancelStep);
    FMI_KLASS_METHOD(fmiGetStatus);
    FMI_KLASS_METHOD(fmiGetRealStatus);
    FMI_KLASS_METHOD(fmiGetIntegerStatus);
    FMI_KLASS_METHOD(fmiGetBooleanStatus);
    FMI_KLASS_METHOD(fmiGetStringStatus);
#undef FMI_KLASS_METHOD
} FmuCoSimulation;

/**
 * Load the library and map the available function to an FMU for model exchange or for co simulation instance.
 *
 * \see dlopen (POSIX)
 * \see LoadLibrary (MSVC)
 */
FmuModelExchange* fmu_wrap_model_exchange_new(const char* lib, const char* name);
FmuCoSimulation* fmu_wrap_co_simulation_new(const char* lib, const char* name);

/**
 * Close the library and release all the available data
 *
 * \see dlclose (POSIX)
 * \see FreeLibrary (MSVC)
 */
void fmu_wrap_model_exchange_delete(FmuModelExchange* instance);
void fmu_wrap_co_simulation_delete(FmuCoSimulation* instance);

/**
 * \return the last error using the fmu_wrap API.
 */
const char* fmu_wrap_error();

/**
 * log some information into the Scilab console
 */
void fmu_callback_logger(fmiComponent c, fmiString instanceName, fmiStatus status, fmiString category, fmiString message, ...);

/**
 * Check that the instance is a valid one.
 *
 * \param instance a pointer to check
 * \return a valid instance of NULL on error
 */
FmuModelExchange* fmu_wrap_model_exchange(void* instance);
FmuCoSimulation* fmu_wrap_co_simulation(void* instance);

/**
 * \fmi Type.
 */
typedef enum {ModelExchange,
              CoSimulation} fmiType;

/**
 * \structure for Xcos simulation.
 */
typedef struct
{
    fmiType type;
    fmiComponent *fmu, *instance;
	fmiReal *states, *nominalStates, *events, *tmpevents, *derivatives;
	fmiBoolean *boolValue, callEventUpdate, stateEvent, timeEvent;
	fmiValueReference *refsStates, *realRefs, *intRefs, *boolRefs;
	fmiEventInfo eventInfo;
	fmiReal oldTime;
} workFMU;

#endif /* FMU_WRAPPER_H */
